var Filterable = function(ZhUtil, Query) {
	var _buildRegExp = function(value) {
		var filterValue = "^" + value + "$";
		var rgx = /\%/;
		while (rgx.test(filterValue)) {
			filterValue = filterValue.replace(rgx, '(.*?)');
		}
		return new RegExp(filterValue, 'i');
	};
	var checkType = function(value) {
		var parsedValue = parseFloat(value);
		if (!isNaN(parsedValue) && parsedValue.toString() === value) {
			value = parsedValue;
		}
		return value;
	};
	var treatArrayValueFilter = function(value) {
		if (typeof(value) === 'string') {
			value = value.split(',');
		}
		if (!Array.isArray(value)) {
			value = [];
		}
		value = value.map(function(item) {
			return ZhUtil.evaluate(item.toString());
		});
		return value;
	};
	var _toString = function(value) {
		if (!Util.isUndefinedOrNull(value)) {
			value = value.toString();
		}
		return value;
	};
	var comparatorFlow = {
		'LIKE': function(item, property, value) {
			var reg = _buildRegExp(ZhUtil.evaluate(value));
			return reg.test(item[property]);
		},
		'LIKE_I': function(item, property, value) {
			return comparatorFlow.LIKE(item, property, value);
		},
		'NOT_LIKE': function(item, property, value) {
			return !comparatorFlow.LIKE(item, property, value);
		},
		'LIKE_ALL': function(item, property, value) {
			var reg = _buildRegExp(ZhUtil.evaluate(value));
			var keys = property === "*" ? Object.keys(item) : property.split('|');
			return keys.some(function(key) {
				return reg.test(item[key]);
			});
		},
		'IN': function(item, property, value) {
			var values = treatArrayValueFilter(value);
			return values.some(function(v) {
				return _toString(item[property]) === v;
			});
		},
		'NOT_IN': function(item, property, value) {
			return !comparatorFlow.IN(item, property, value);
		},
		'BETWEEN': function(item, property, value) {
			value = treatArrayValueFilter(value);
			var result = false;
			if (Array.isArray(value)) {
				var propertyValue = checkType(_toString(item[property]));
				result = propertyValue >= checkType(value[0]) && propertyValue <= checkType(value[1]);
			}
			return result;
		},
		'NOT_BETWEEN': function(item, property, value) {
			return !comparatorFlow.BETWEEN(item, property, value);
		},
		'=': function(item, property, value) {
			return _toString(item[property]) === ZhUtil.evaluate(value.toString());
		},
		'<>': function(item, property, value) {
			return _toString(item[property]) !== ZhUtil.evaluate(value.toString());
		},
		'<': function(item, property, value) {
			return checkType(item[property]) < checkType(ZhUtil.evaluate(value.toString()));
		},
		'<=': function(item, property, value) {
			return checkType(item[property]) <= checkType(ZhUtil.evaluate(value.toString()));
		},
		'>': function(item, property, value) {
			return checkType(item[property]) > checkType(ZhUtil.evaluate(value.toString()));
		},
		'>=': function(item, property, value) {
			return checkType(item[property]) >= checkType(ZhUtil.evaluate(value.toString()));
		},
		'IS_NULL': function(item, property, value) {
			return Util.isNull(item[property]);
		},
		'IS_NOT_NULL': function(item, property, value) {
			return !Util.isNull(item[property]);
		},
		'ORDER_BY': function(item, property, value) {
			return true;
		}
	};
	this.applyFilters = function(item, filters) {
		var result = true;
		if (item && filters && filters.length > 0) {
			result = filters.every(function(filter) {
				return comparatorFlow[filter.operator](item, filter.name, filter.value);
			});
		}
		return result;
	};
	this.processFilter = function(data, filters) {
		var filteredValues = data;
		for (var i in filters) {
			if (filters.hasOwnProperty(i)) {
				filteredValues = this.queryFilter(filteredValues, filters[i]);
			}
		}
		return filteredValues;
	};
	this.queryFilter = function(data, queryParams) {
		var filteredData = data;
		if (!queryParams.value && queryParams.value !== false) {
			ZhLogger.error(queryParams.value + " is an invalid name for a filter.");
			return [];
		}
		switch (queryParams.operator) {
			case Query.LIKE:
				filteredData = this.likeFilter(filteredData, queryParams);
				break;
			case Query.NOT_LIKE:
				filteredData = this.notLikeFilter(filteredData, queryParams);
				break;
			case Query.LIKE_ALL:
				filteredData = this.likeAllFilter(filteredData, queryParams);
				break;
			case Query.IN:
				filteredData = this.inFilter(filteredData, queryParams);
				break;
			case Query.NOT_IN:
				filteredData = this.notInFilter(filteredData, queryParams);
				break;
			case Query.BETWEEN:
				filteredData = this.betweenFilter(filteredData, queryParams);
				break;
			case Query.NOT_BETWEEN:
				filteredData = this.notBetweenFilter(filteredData, queryParams);
				break;
			case Query.EQ:
				filteredData = this.equalFilter(filteredData, queryParams);
				break;
			case Query.NEQ:
				filteredData = this.notEqualFilter(filteredData, queryParams);
				break;
			case Query.LT:
				filteredData = this.lessFilter(filteredData, queryParams);
				break;
			case Query.LTE:
				filteredData = this.lessEqualFilter(filteredData, queryParams);
				break;
			case Query.GT:
				filteredData = this.moreFilter(filteredData, queryParams);
				break;
			case Query.GTE:
				filteredData = this.moreEqualFilter(filteredData, queryParams);
				break;
			case Query.IS_NULL:
				filteredData = this.isNullFilter(filteredData, queryParams);
				break;
			case Query.IS_NOT_NULL:
				filteredData = this.isNotNullFilter(filteredData, queryParams);
				break;
			default:
				break;
		}
		return filteredData;
	};
	this.inFilter = function(data, params) {
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(params.value)) {
				result = comparatorFlow[Query.IN](item, params.name, params.value);
			}
			return result;
		});
	};
	this.notInFilter = function(data, params) {
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(params.value)) {
				result = comparatorFlow[Query.NOT_IN](item, params.name, params.value);
			}
			return result;
		});
	};
	this.likeFilter = function(data, params) {
		var valEvaluated = ZhUtil.evaluate(_toString(params.value));
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(valEvaluated)) {
				result = comparatorFlow[Query.LIKE](item, params.name, valEvaluated);
			}
			return result;
		});
	};
	this.notLikeFilter = function(data, params) {
		var valEvaluated = ZhUtil.evaluate(_toString(params.value));
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(valEvaluated)) {
				result = comparatorFlow[Query.NOT_LIKE](item, params.name, valEvaluated);
			}
			return result;
		});
	};
	this.likeAllFilter = function(data, params) {
		var valEvaluated = ZhUtil.evaluate(_toString(params.value));
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(valEvaluated)) {
				result = comparatorFlow[Query.LIKE_ALL](item, params.name, valEvaluated);
			}
			return result;
		});
	};
	this.equalFilter = function(data, params) {
		var valEvaluated = ZhUtil.evaluate(_toString(params.value));
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(valEvaluated)) {
				result = comparatorFlow[Query.EQ](item, params.name, valEvaluated);
			}
			return result;
		});
	};
	this.notEqualFilter = function(data, params) {
		var valEvaluated = ZhUtil.evaluate(_toString(params.value));
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(valEvaluated)) {
				result = comparatorFlow[Query.NEQ](item, params.name, valEvaluated);
			}
			return result;
		});
	};
	this.betweenFilter = function(data, params) {
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(params.value)) {
				result = comparatorFlow[Query.BETWEEN](item, params.name, params.value);
			}
			return result;
		});
	};
	this.notBetweenFilter = function(data, params) {
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(params.value)) {
				result = comparatorFlow[Query.NOT_BETWEEN](item, params.name, params.value);
			}
			return result;
		});
	};
	this.lessFilter = function(data, params) {
		var valEvaluated = ZhUtil.evaluate(_toString(params.value));
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(valEvaluated)) {
				result = comparatorFlow[Query.LT](item, params.name, valEvaluated);
			}
			return result;
		});
	};
	this.lessEqualFilter = function(data, params) {
		var valEvaluated = ZhUtil.evaluate(_toString(params.value));
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(valEvaluated)) {
				result = comparatorFlow[Query.LTE](item, params.name, valEvaluated);
			}
			return result;
		});
	};
	this.moreFilter = function(data, params) {
		var valEvaluated = ZhUtil.evaluate(_toString(params.value));
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(valEvaluated)) {
				result = comparatorFlow[Query.GT](item, params.name, valEvaluated);
			}
			return result;
		});
	};
	this.moreEqualFilter = function(data, params) {
		var valEvaluated = ZhUtil.evaluate(_toString(params.value));
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name]) && !Util.isUndefined(valEvaluated)) {
				result = comparatorFlow[Query.GTE](item, params.name, valEvaluated);
			}
			return result;
		});
	};
	this.isNullFilter = function(data, params) {
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name])) {
				result = comparatorFlow[Query.IS_NULL](item, params.name);
			}
			return result;
		});
	};
	this.isNotNullFilter = function(data, params) {
		return data.filter(function(item) {
			var result = false;
			if (!Util.isUndefined(item[params.name])) {
				result = comparatorFlow[Query.IS_NOT_NULL](item, params.name);
			}
			return result;
		});
	};

	this.extractFiltersByOperator = function(filters, operator) {
		var extractedFilters = [];
		for (var i in filters) {
			if (filters.hasOwnProperty(i) && filters[i].operator === operator) {
				extractedFilters = extractedFilters.concat(filters.splice(i, 1));
			}
		}
		return extractedFilters;
	};

	this.applyOrder = function(data, orderFilters) {
		if (orderFilters && orderFilters.length) {
			var sortPredicate = [];
			var direction;
			orderFilters.forEach(function(order) {
				order.name.forEach(function(orderFilter) {
					direction = orderFilter.direction === Query.ORDER_DESC ? '-' : '+';
					sortPredicate.push(direction + orderFilter.name);
				});
			});
			data = ZhUtil.orderBy(data, sortPredicate);
		}
		return data;
	};

};